# Zope imports
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass

# CMF imports
from Products.CMFCore.utils import UniqueObject#, getToolByName

# Implementation imports
import amazon

# Available attributes on amazon search results, with descriptions
attrs = {'Asin': 'Amazon ID ("ASIN" number) of this item',
         'Authors': 'list of authors',
         'Availability': '"available", etc.',
         #'BrowseList': 'list of related categories',
         'Catalog': 'catalog type ("Book", etc)',
         'CollectiblePrice': '?, format "$34.95"',
         'ImageUrlLarge': 'URL of large image of this item',
         'ImageUrlMedium':  'URL of medium image of this item',
         'ImageUrlSmall': 'URL of small image of this item',
         'Isbn': 'ISBN number',
         'ListPrice': 'list price, format "$34.95"',
         #'Lists': 'list of ListMania lists that include this item',
         'Manufacturer': 'manufacturer',
         'Media': 'media ("Paperback", "Audio CD", etc)',
         'NumMedia': 'number of different media types in which this item is available',
         'OurPrice': 'Amazon price, format "$24.47"',
         'ProductName': 'name of this item',
         'ReleaseDate': 'release date, format "09 April, 1999"',
         #'Reviews': 'reviews (AvgCustomerRating, plus list of CustomerReview with Rating, Summary, Content)',
         'SalesRank': 'sales rank (integer)',
         #'SimilarProducts': 'list of Product, which is ASIN number',
         'ThirdPartyNewPrice': '?, format "$34.95"',
         'URL': 'URL of this item' }

# Marker for __getattr__ and get calls
_default = ''

class ZopeBag:

    security = ClassSecurityInfo()
    security.declareObjectPublic()
    security.setDefaultAccess("allow")

    def __init__(self, bag):
        self._bag = bag

    def __getattr__(self, name, default=_default):
        """ """
        if name == 'Authors':
            return getattr(self._bag.Authors, 'Author', default)
        elif name in attrs.keys():
            attr = getattr(self._bag, name, default)
            if isinstance(attr, amazon.Bag):
                return ZopeBag(attr)
            else:
                return attr
        elif default is not _default:
            return default
        else:
            raise AttributeError(name)

    security.declarePublic('keys')
    def keys(self):
        """ """
        return attrs.keys()

    def __getitem__(self, key):
        """ """
        try:
            return getattr(self, key)
        except AttributeError:
            raise KeyError(key)

    def get(self, key, default=_default):
        """ """
        try:
            return getattr(self, key, _default)
        except AttributeError:
            raise KeyError(key)


class BadLicenseKey(Exception):
    pass


class AmazonTool(PropertyManager, UniqueObject, SimpleItem):
    """CMF wrapper around the PyAmazon library.
    http://www.josephson.org/projects/pyamazon/.
    """

    id = 'amazon_tool'
    meta_type = 'Amazon Tool'
    security = ClassSecurityInfo()

    # Default value for the license_key and locale
    license_key = ''
    locale = 'us'

    _properties = (
        {'id':'title',       'type':'string',    'mode':'w'},
        {'id':'license_key', 'type':'string',    'mode':'w'},
        {'id':'locale',      'type':'selection', 'mode':'w', 'select_variable':'getLocales'},
        )

    manage_options=(
        {'label':'Properties', 'action':'manage_propertiesForm',
         'help':('OFSP','Properties.stx')},
        )

    security.declareProtected('Amazon Tool: Search Amazon', 'searchByASIN')
    def searchByASIN(self, asin):
        """Return details associated with the ASIN (Amazon's name for ISBN apparently).
        """
        if self.license_key:
            result_bags = amazon.searchByASIN(asin, license_key=self.license_key, locale=self.locale)
            results = [ ZopeBag(bag) for bag in result_bags ]
            return results
        else:
            raise BadLicenseKey(self.license_key)

    security.declarePublic('getResultAttributes')
    def getResultAttributes(self):
        """ """
        d = {}
        for key in attrs.keys():
            d[key] = attrs[key]
        return d

    def getLocales(self):
        """ """
        return amazon._supportedLocales.keys()


InitializeClass(AmazonTool)
InitializeClass(ZopeBag)
