# Zope imports
from AccessControl import ClassSecurityInfo
from DateTime.DateTime import DateTime

# CMF imports
from Products.CMFCore.utils import getToolByName

# Archetypes imports
from Products.Archetypes.public import BaseSchema, Schema
from Products.Archetypes.public import StringField, LinesField, IntegerField, ReferenceField
from Products.Archetypes.public import TextField, DateTimeField, BooleanField
from Products.Archetypes.public import LinesWidget, StringWidget, CalendarWidget, BooleanWidget
from Products.Archetypes.public import RichWidget, IntegerWidget, SelectionWidget, ReferenceWidget
from Products.Archetypes.public import BaseContent, registerType
from Products.Archetypes.Marshall import PrimaryFieldMarshaller
from Products.Archetypes.interfaces.referenceable import IReferenceable

# Product imports
from config import PROJECTNAME, perms, READ_STATUS_VOCAB
from YearAttribute import YearField, YearWidget


journal_schema = BaseSchema + Schema((
    StringField('articleTitle',
                searchable=1,
                widget=StringWidget(label='Title of the article'),
                ),
    LinesField('authors',
                searchable=1,
                widget=LinesWidget(label='Authors'),
                ),
    StringField('journalTitle',
                searchable=1,
                widget=StringWidget(label='Title of the journal'),
                ),
    StringField('journalISSN',
                searchable=1,
                widget=StringWidget(label='ISSN of the journal'),
                ),
    #DateTimeField('publishingDate',
    #            searchable=1,
    #            widget=CalendarWidget(label='Publishing date'),
    #            ),
    YearField('publicationYear',
              searchable=1,
              widget=YearWidget(label='Year of publication'),
              ),
    IntegerField('volume',
                searchable=1,
                widget=IntegerWidget(label='Volume of the journal'),
                ),
    IntegerField('volumeNumber',
                searchable=1,
                widget=IntegerWidget(label='Number within volume of the journal'),
                ),
    IntegerField('startPage',
                searchable=1,
                widget=IntegerWidget(label='Start page number'),
                ),
    IntegerField('endPage',
                searchable=1,
                widget=IntegerWidget(label='End page number'),
                ),
    TextField('abstract',
                searchable=1,
                widget=RichWidget(label='Abstract'),
                ),
    ),
                              marshall=PrimaryFieldMarshaller(),
                              )


book_schema = BaseSchema + Schema((
    StringField('isbn',
                searchable=1,
                widget=StringWidget(label='ISBN'),
                ),
    LinesField('authors',
                searchable=1,
                widget=LinesWidget(label='Authors'),
                ),
    StringField('smallImageUrl',
                searchable=1,
                widget=StringWidget(label='URL to an associated small image'),
                ),
    StringField('mediumImageUrl',
                searchable=1,
                widget=StringWidget(label='URL to an associated image'),
                ),
    StringField('largeImageUrl',
                searchable=1,
                widget=StringWidget(label='URL to an associated large image'),
                ),
    StringField('amazonPrice',
                searchable=1,
                widget=StringWidget(label='Price on Amazon'),
                ),
    StringField('amazonUrl',
                searchable=1,
                widget=StringWidget(label='URL to Amazon page'),
                validators=['isURL',]
                ),
    TextField('synopsis',
                searchable=1,
                widget=RichWidget(label='Book synopsis'),
                ),
    DateTimeField('publicationDate',
                searchable=1,
                widget=CalendarWidget(label='Release date of book'),
                ),
    StringField('readStatus',
                searchable=1,
                default='unread',
                vocabulary=READ_STATUS_VOCAB,
                widget=SelectionWidget(label='How much has been read?'),
                ),
    BooleanField('isPurchased',
                searchable=1,
                widget=BooleanWidget(label='Is already purchased?'),
                ),
    StringField('publisherName',
                searchable=1,
                widget=StringWidget(label='Name of the publisher'),
                ),
    StringField('publisherLocation',
                searchable=1,
                widget=StringWidget(label='Location of the publisher'),
                ),
    ),
                              marshall=PrimaryFieldMarshaller(),
                              )


book_chapter_schema = BaseSchema + Schema((
    StringField('chapter',
                searchable=1,
                widget=StringWidget(label='Chapter name or number'),
                ),
    LinesField('chapterAuthors',
                searchable=1,
                widget=LinesWidget(label='Authors of the chapter'),
                ),
    ReferenceField('book',
                searchable=1,
                #vocabulary='getBookRefs',
                allowed_types=['BookRef'],
                widget=ReferenceWidget(label='Book in which the chapter appears'),
                ),
    IntegerField('startPage',
                searchable=1,
                widget=IntegerWidget(label='Start page number'),
                ),
    IntegerField('endPage',
                searchable=1,
                widget=IntegerWidget(label='End page number'),
                ),
    TextField('chapterAbstract',
                searchable=1,
                widget=RichWidget(label='Abstract of the chapter'),
                ),
    StringField('readStatus',
                searchable=1,
                default='unread',
                vocabulary=READ_STATUS_VOCAB,
                widget=SelectionWidget(label='How much has been read?'),
                ),
    ),
                              marshall=PrimaryFieldMarshaller(),
                              )


class JournalRef(BaseContent):
    """ """
    schema = journal_schema
    __implements__ = BaseContent.__implements__ + (IReferenceable,)


class BookRef(BaseContent):
    """ """
    schema = book_schema
    __implements__ = BaseContent.__implements__ + (IReferenceable,)
    security = ClassSecurityInfo()

    actions = ({
        'id': 'setDetails',
        'name': 'Set Details from Amazon',
        'action': 'string:${object_url}/setDetailsFromISBN',
        'permissions': (perms.MODIFY,)
        },)

    security.declareProtected(perms.MODIFY, 'setDetailsFromISBN')
    def setDetailsFromISBN(self, isbn=None, REQUEST=None):
        """Get details from Amazon using the AmazonTool.
        If asin == None, try to use the asin (isbn) already on the object.
        """
        atool = getToolByName(self, 'amazon_tool')
        if isbn is None:
            isbn = self.getIsbn().strip()
        if not isbn:
            raise Exception('Bad ISBN')
        result = atool.searchByASIN(asin=isbn)[0]
        self.setTitle(result.ProductName)
        self.setAuthors(result.Authors)
        self.setSmallImageUrl(result.ImageUrlSmall)
        self.setMediumImageUrl(result.ImageUrlMedium)
        self.setLargeImageUrl(result.ImageUrlLarge)
        self.setAmazonPrice(result.OurPrice)
        #self.setPublicationDate(DateTime(result.ReleaseDate))
        # result.ReleaseDate looks like u'March, 2004', which confuses DateTime
        release_date = '1 %s' % result.ReleaseDate
        self.setPublicationDate(release_date)
        self.setAmazonUrl(result.URL)
        self.setPublisherName(result.Manufacturer)

        if REQUEST is not None:
            self.REQUEST.RESPONSE.redirect(self.absolute_url())


class BookChapterRef(BaseContent):
    """ """
    schema = book_chapter_schema
    __implements__ = BaseContent.__implements__ + (IReferenceable,)


registerType(JournalRef, PROJECTNAME)
registerType(BookRef, PROJECTNAME)
registerType(BookChapterRef, PROJECTNAME)