# Zope imports
#from OFS.ObjectManager import ObjectManager
from OFS.Folder import Folder
#from OFS.PropertyManager import PropertyManager
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass

# CMF imports
from Products.CMFCore.utils import UniqueObject#, getToolByName

# Product imports
from config import perms, BTOOL_DIR

# Implementation imports


class BibliozTool(UniqueObject, Folder):
    """Tool for rendering Biblioz references in various citation styles.
    """

    id = 'biblioz_tool'
    meta_type = 'Biblioz Tool'
    security = ClassSecurityInfo()

    default_format = 'harvard'

    _properties = (
        {'id':'title', 'type':'string', 'mode':'w'},
        {'id':'default_format', 'type':'selection', 'mode':'w', 'select_variable':'getFormats'},
        )

    security.declareProtected(perms.USE_BIBLIOZ_TOOL, 'renderCitation')
    def renderCitation(self, biblioz, format=None):
        """ """
        if format is None:
            format = self.getProperty('default_format')
        script = self._getTemplateScript(biblioz.meta_type, format)
        return script(biblioz)

    def _getTemplateScript(self, meta_type, format):
        # Look for a ZODB implementation
        if hasattr(self, format):
            format_folder = self[format]
            if hasattr(format_folder, meta_type):
                return format_folder[meta_type]
        # Look for a default (FSDirView) implementation
        return self[BTOOL_DIR][format][meta_type]


    security.declareProtected(perms.USE_BIBLIOZ_TOOL, 'getFormats')
    def getFormats(self):
        """ """
        return self.objectIds(['Folder'])

    def manage_afterAdd(self, item, container):
        """ """
        self.manage_addProduct['CMFCore'].manage_addDirectoryView('Biblioz/%s' % BTOOL_DIR)

InitializeClass(BibliozTool)
